"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const observable_1 = require("@reactive-js/observable");
class AsyncIteratorResourceImpl {
    constructor(dispatcher, disposable, observable) {
        this.dispatcher = dispatcher;
        this.disposable = disposable;
        this.observable = observable;
    }
    get isDisposed() {
        return this.disposable.isDisposed;
    }
    add(disposable, ...disposables) {
        this.disposable.add(disposable, ...disposables);
    }
    dispatch(req) {
        this.dispatcher(req);
    }
    dispose() {
        this.disposable.dispose();
    }
    remove(disposable, ...disposables) {
        this.disposable.remove(disposable, ...disposables);
    }
    subscribe(subscriber) {
        this.observable.subscribe(subscriber);
    }
}
const liftImpl = (operator, mapper) => iterator => {
    const [observable, dispatcher, disposable] = iterator instanceof AsyncIteratorResourceImpl
        ? [iterator.observable, iterator.dispatcher, iterator.disposable]
        : [iterator, (req) => iterator.dispatch(req), iterator];
    const pipedObservable = operator !== undefined ? observable_1.pipe(observable, operator) : observable;
    const mappedDispatcher = mapper !== undefined ? req => dispatcher(mapper(req)) : dispatcher;
    return new AsyncIteratorResourceImpl(mappedDispatcher, disposable, pipedObservable);
};
exports.lift = (operator) => liftImpl(operator, undefined);
exports.liftReq = (mapper) => liftImpl(undefined, mapper);
function pipe(src, ...operators) {
    return operators.reduce((acc, next) => next(acc), src);
}
exports.pipe = pipe;
exports.createEvent = () => {
    const subject = observable_1.createSubject();
    const dispatcher = (req) => subject.next(req);
    return new AsyncIteratorResourceImpl(dispatcher, subject, subject);
};
exports.createStateStore = (initialState, scheduler, equals) => {
    const subject = observable_1.createSubject();
    const dispatcher = (req) => subject.next(req);
    const observable = observable_1.pipe(subject, observable_1.scan((acc, next) => next(acc), initialState), observable_1.startWith(initialState), observable_1.distinctUntilChanged(equals), observable_1.share(scheduler, 1));
    subject.add(observable_1.connect(observable, scheduler));
    return new AsyncIteratorResourceImpl(dispatcher, subject, observable);
};
exports.concatAll = (maxBufferSize = Number.MAX_SAFE_INTEGER) => exports.lift(observable_1.concatAll(maxBufferSize));
exports.distinctUntilChanged = (equals) => exports.lift(observable_1.distinctUntilChanged(equals));
exports.endWith = (value, ...values) => exports.lift(observable_1.endWith(value, ...values));
exports.exhaust = () => exports.lift(observable_1.exhaust());
exports.ignoreElements = () => exports.lift(observable_1.ignoreElements());
exports.keep = (predicate) => exports.lift(observable_1.keep(predicate));
exports.map = (mapper) => exports.lift(observable_1.map(mapper));
exports.mergeAll = (options) => exports.lift(observable_1.mergeAll(options));
exports.observe = (observer) => exports.lift(observable_1.observe(observer));
exports.onComplete = (onComplete) => exports.lift(observable_1.onComplete(onComplete));
exports.onError = (onError) => exports.lift(observable_1.onError(onError));
exports.onNext = (onNext) => exports.lift(observable_1.onNext(onNext));
exports.repeat = (predicate) => exports.lift(observable_1.repeat(predicate));
exports.retry = (predicate) => exports.lift(observable_1.retry(predicate));
exports.scan = (scanner, initialValue) => exports.lift(observable_1.scan(scanner, initialValue));
exports.share = (scheduler, replayCount) => exports.lift(observable_1.share(scheduler, replayCount));
exports.startWith = (value, ...values) => exports.lift(observable_1.startWith(value, ...values));
exports.subscribeOn = (scheduler) => exports.lift(observable_1.subscribeOn(scheduler));
exports.switchAll = () => exports.lift(observable_1.switchAll());
exports.take = (count) => exports.lift(observable_1.take(count));
exports.takeLast = (count) => exports.lift(observable_1.takeLast(count));
exports.takeWhile = (predicate) => exports.lift(observable_1.takeWhile(predicate));
exports.throttle = (durationSelector) => exports.lift(observable_1.throttle(durationSelector));
exports.throttleTime = (duration) => exports.lift(observable_1.throttleTime(duration));
exports.throttleFirst = (durationSelector) => exports.lift(observable_1.throttleFirst(durationSelector));
exports.throttleFirstTime = (duration) => exports.lift(observable_1.throttleFirstTime(duration));
exports.throttleLast = (durationSelector) => exports.lift(observable_1.throttleLast(durationSelector));
exports.throttleLastTime = (duration) => exports.lift(observable_1.throttleLastTime(duration));
exports.timeout = (duration) => exports.lift(observable_1.timeout(duration));
exports.withLatestFrom = (other, selector) => exports.lift(observable_1.withLatestFrom(other, selector));
//# sourceMappingURL=index.js.map